//----------------------------------------------------------------------------
// C main line
//----------------------------------------------------------------------------
#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules
#include <stdlib.h>
#include "motorcontroller.h"
#include "rx8_1.h"
#include <psocgpioint.h>

// Global Variables

int 	TimeSlot = 0;			// Scheduler variable
int 	SliceCounter = 0;		// Scheduler variable

// Main Program

void main()
{

// Variables

	CmdStruct	Command;
	WheelStruct	EastWheel;
	WheelStruct WestWheel;

	BYTE		bRxStatus;
	BYTE		bTxStatus;
	BYTE		CmdRxByte = 0;			// Rx interrupt Variables
	BYTE		StsTxByte = 0;			// Tx interrupt Variables
	BYTE		LS_Mask = 0;
	BYTE		R_LS_Mask = 0;
	BYTE		L_LS_Mask = 0;
	BYTE		Mask = 0;
	BOOL		Key;
	BOOL		printstr;
	BOOL		Stop;
	BOOL		PSoC_Select;
	BOOL		Weigh;
	
	char 	    str[17];			            // One line of characters for LCD
	char		blank[] = "                ";	// Clear a line of LCD
	
	int 		pwr[4];
	int			cnt;
	

// Initialization

	initPSoC(); 						// Init PSoC functions
	StopMotor(BOTH);
	StsTxByte |= PSOCSTOP;
	
	EastWheel.id = EAST;				// Initialize Wheel Structs
	WestWheel.id = WEST;
	Command.Status = NEW;				// Initialize Command Structs
	Command.Velocity = 0;
	Command.END_SENSOR = FALSE;
	
	Stop = FALSE;
	PSoC_Select = FALSE;
	Weigh = FALSE;
	
	pwr[0] = 1;
	pwr[1] = 2;
	pwr[2] = 4;
	pwr[3] = 8;

	
	while(TRUE)
	{
// Do this every 10 ms
// Check to see if we got a byte from XBC.  If so check to see
// if data or command byte.  Then go ahead and process the byte.

		if (TimeSlot & 2)
		{
			TimeSlot ^= 2;

			if (bRxStatus = RX8_1_bReadRxStatus() & RX8_RX_COMPLETE)
			{
				CmdRxByte = RX8_1_bReadRxData();
								
				if (CmdRxByte & EW_PSOC)
				{
					PSoC_Select = TRUE;
					TX8_1_Start(TX8_PARITY_NONE);				
					ProcessRxByte(CmdRxByte, &Command);
					StsTxByte |= EW_PSOC;
					StsTxByte &= PSOCSTART;
					StsTxByte |= STSWORD;
					
					if (Command.Velocity == 0)
					{
						Weigh = TRUE;
						Stop = TRUE;

					}
				}		
				
				bRxStatus = 0;
				RX8_1_CmdReset();
			}
			
			if (PSoC_Select && (CmdRxByte & ALIGN) && (Command.Velocity > 0))
			{
				LS_Mask = (PRT2DR | IS_LN_MSK);
				
				switch (LS_Mask)
				{
					case LS_MSK6:
						Stop = TRUE;
					break;
					
					case LS_MSK7:
						Stop = TRUE;
					break;
					
					case LS_MSK8:
						Stop = TRUE;
					break;
					
					case LS_MSK9:
						Stop = TRUE;
					break;
					
					case IS_LN_MSK:
						Stop = TRUE;
					break;
					
					default:
						Stop = FALSE;
				}
			}
			
			else if (PSoC_Select && (CmdRxByte & IS_DT) && (Command.Velocity > 0))
			{
				LS_Mask = (PRT2DR | IS_LN_MSK);
				
				switch (LS_Mask)
				{
					case LS_MSK1:
						Stop = TRUE;
					break;
					
					case LS_MSK2:
						Stop = TRUE;
					break;
					
					case LS_MSK3:
						Stop = TRUE;
					break;
					
					case LS_MSK4:
						Stop = TRUE;
					break;
					
					case LS_MSK5:
						Stop = TRUE;
					break;
					
					case LS_MSK6:
						Stop = TRUE;
					break;
					
					case LS_MSK7:
						Stop = TRUE;
					break;
					
					case LS_MSK8:
						Stop = TRUE;
					break;
					
					case LS_MSK9:
						Stop = TRUE;
					break;
					
					case LS_MSK10:
						Stop = TRUE;
					break;
					
					case LS_MSK11:
						Stop = TRUE;
					break;
					
					case LS_MSK12:
						Stop = TRUE;
					break;
					
					case LS_MSK13:
						Stop = TRUE;
					break;
					
					case LS_MSK14:
						Stop = TRUE;
					break;
					
					case LS_MSK15:
						Stop = TRUE;
					break;
					
					case LS_MSK16:
						Stop = TRUE;
					break;
					
					case LS_MSK17:
						Stop = TRUE;
					break;
					
					case LS_MSK18:
						Stop = TRUE;
					break;
					
					case IS_LN_MSK:
						Stop = TRUE;
					break;
					
					default:
						LS_Mask = (PRT2DR & IS_LN_MSK);
						if (!LS_Mask)
						{
							Stop = TRUE;
						}
						
						else
						{
							Stop = FALSE;
						}
				}
			}				
						
			if ((PSoC_Select && Stop) || (PSoC_Select && Command.Velocity == 0))
			{
				StopMotor(BOTH);
				
	       		Command.Velocity = 0;
	       		Command.VP6 = 0;
	   			Command.VP5 = 0;
	   			Command.VP4 = 0;
	   			Command.VP3 = 0;
	   			Command.VP2 = 0;
	   			Command.VP1 = 0;
	   			Command.VS1 = 0;
	   			Command.VS2 = 0;
	   			Command.VS3 = 0;
	   			Command.VS4 = 0;
	   			Command.VS5 = 0;
	   			Command.VS6 = 0;
	       		
	       		if (Stop)
	       		{
	       			StsTxByte |= PSOCSTOP;
	       			Stop = FALSE;
	       		}
	       			       		
	       		if (Command.Offline)
	       		{
	       	
	       			StsTxByte |= LINELEFT;
	       		}
	       		
	       		else
	       		{
	       			StsTxByte &= LINERIGHT;
	       		}
	       		
	       		if (Command.Center)
	       		{
	       			StsTxByte |= CENTER;
	       		}
	       		
	       		if (PSoC_Select)
	       		{
	       			TX8_1_SendData(StsTxByte);
	       			TX8_2_SendData(0x94);
					TX8_2_PutSHexByte(StsTxByte);
					Weigh = FALSE;
					PSoC_Select = FALSE;
	       		}
				
				if (bTxStatus = bTX8_1_ReadTxStatus() & TX8_1_TX_COMPLETE)
				{
					TX8_1_Stop();
					bTxStatus = 0;
				}
			}		
		}// end 10 ms timeslot

//
// Do this every 20 ms
//
		if (TimeSlot & 4)
		{
			TimeSlot ^= 4;
			
			LS_Mask = (PRT2DR | IS_LN_MSK);
			LS_Mask = ~LS_Mask;
			
/*			TX8_2_SendData(0x84);
			TX8_2_PutSHexByte(LS_Mask);
		
			while(!bTxStatus)
			{
				bTxStatus = bTX8_2_ReadTxStatus() & TX8_2_TX_COMPLETE;
			}
			
			bTxStatus = 0;*/
			
			cnt = 3;
			R_LS_Mask = LS_Mask >> 4;
			L_LS_Mask = 0;
			
			while(cnt >= 0)
			{
				Mask = 0;
				LS_Mask = LS_Mask >> 1;
				Mask = (LS_Mask & IS_LN_MSK);
				Mask = Mask * pwr[cnt];
				L_LS_Mask = (L_LS_Mask | Mask);
				cnt--;
			}				
			
			if (R_LS_Mask < L_LS_Mask)
			{
				Command.Offline = LEFT;
			}
			
			else
			{
				Command.Offline = RIGHT;
			}
			
/*			TX8_2_SendData(0x80);
			TX8_2_PutSHexByte(R_LS_Mask);
			
			while(!bTxStatus)
			{
				bTxStatus = bTX8_2_ReadTxStatus() & TX8_2_TX_COMPLETE;
			}
			
			bTxStatus = 0;
			
			TX8_2_SendData(0x94);
			TX8_2_PutSHexByte(L_LS_Mask);
			
			while(!bTxStatus)
			{
				bTxStatus = bTX8_2_ReadTxStatus() & TX8_2_TX_COMPLETE;
			}
			
			bTxStatus = 0;
						
			TX8_2_SendData(0x98);
			TX8_2_PutSHexByte(PRT2DR);*/			
		}// end 20 ms timeslot

// Do this every 40 ms (the sample period is 40 ms)

		if (TimeSlot & 8)
		{
			TimeSlot ^= 8;
			
			if (PSoC_Select)
			{
				ControlWheel(&Command, &WestWheel);
				ControlWheel(&Command, &EastWheel);	
			}
			
		}// end 40 ms timeslot

// Do this every 160 ms

		if (TimeSlot & 32)
		{
			TimeSlot ^= 32;
		}// end 160 ms timeslot

// Update the LCD screen once each 1.28 seconds

		if (TimeSlot & 256)
		{
			TimeSlot ^= 256;
			
		}// end 1280 ms timeslot
	}// end while loop
}// end of main program



////////////////////////////////////////////////////////////////////////

// Timer interrupt routine (every 5 ms)
#pragma interrupt_handler Timer16_1_ISR
#pragma nomac

void Timer16_1_ISR()
{
	int temp;

	temp = SliceCounter++;
	TimeSlot |= ((temp ^ SliceCounter) + 1);
}