#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules
#include <stdlib.h>
#include "motorcontroller.h"

void ProcessRxByte(BYTE RxByte, CmdStruct * Cmd)
{
	BYTE	VelocityCode;

	Cmd->MotionType = ((RxByte & 0x20) >> 5);
	VelocityCode = ((RxByte & 0x18) >> 3);
	if (RxByte & LF_ON)
		Cmd->LineFollow = 1;
		else
		Cmd->LineFollow = 0;
	

	switch (VelocityCode)
	{
	   	case 0x0:
	   		Cmd->Velocity = 0;
	   		Cmd->VP6 = 0;
	   		Cmd->VP5 = 0;
	   		Cmd->VP4 = 0;
	   		Cmd->VP3 = 0;
	   		Cmd->VP2 = 0;
	   		Cmd->VP1 = 0;
	   		Cmd->VS1 = 0;
	   		Cmd->VS2 = 0;
	   		Cmd->VS3 = 0;
	   		Cmd->VS4 = 0;
	   		Cmd->VS5 = 0;
	   		Cmd->VS6 = 0;
	    break;

	   	case 0x1:
	   		Cmd->Velocity = SLOW;
	   		Cmd->VP6 = SLOWP6;
	   		Cmd->VP5 = SLOWP5;
	   		Cmd->VP4 = SLOWP4;
	   		Cmd->VP3 = SLOWP3;
	   		Cmd->VP2 = SLOWP2;
	   		Cmd->VP1 = SLOWP1;
	   		Cmd->VS1 = SLOWS1;
	   		Cmd->VS2 = SLOWS2;
	   		Cmd->VS3 = SLOWS3;
	   		Cmd->VS4 = SLOWS4;
	   		Cmd->VS5 = SLOWS5;
	   		Cmd->VS6 = SLOWS6;
	    break;

	   	case 0x2:
	   		Cmd->Velocity = MEDIUM;
	   		Cmd->VP6 = MEDIUMP6;
	   		Cmd->VP5 = MEDIUMP5;
	   		Cmd->VP4 = MEDIUMP4;
	   		Cmd->VP3 = MEDIUMP3;
	   		Cmd->VP2 = MEDIUMP2;
	   		Cmd->VP1 = MEDIUMP1;
	   		Cmd->VS1 = MEDIUMS1;
	   		Cmd->VS2 = MEDIUMS2;
	   		Cmd->VS3 = MEDIUMS3;
	   		Cmd->VS4 = MEDIUMS4;
	   		Cmd->VS5 = MEDIUMS5;
	   		Cmd->VS6 = MEDIUMS6;
	    break;

	   	case 0x3:
	   		Cmd->Velocity = FAST;
	   		Cmd->VP6 = FASTP6;
	   		Cmd->VP5 = FASTP5;
	   		Cmd->VP4 = FASTP4;
	   		Cmd->VP3 = FASTP3;
	   		Cmd->VP2 = FASTP2;
	   		Cmd->VP1 = FASTP1;
	   		Cmd->VS1 = FASTS1;
	   		Cmd->VS2 = FASTS2;
	   		Cmd->VS3 = FASTS3;
	   		Cmd->VS4 = FASTS4;
	   		Cmd->VS5 = FASTS5;
	   		Cmd->VS6 = FASTS6;
	    break;

	   	default:
	   		Cmd->Velocity = 0;
	}//End switch

	if (Cmd->MotionType == TRANS)
	{
		if(RxByte & 0x04)
		{
			Cmd->Direction = BWD;
		}

		else
		{
			Cmd->Direction = FWD;
		}
	}

	Cmd->Status = NEW;

	return;

}// End ProcessRxByte


/////////////////////////////////////////////////////////////////////////////

void ControlWheel(CmdStruct * Cmd, WheelStruct * Wheel)
{
	BYTE	LS_MASK = 0;
	
	if (Cmd->Status == NEW)
	{
		if (Cmd->MotionType == TRANS)
		{
			if(Wheel->id == WEST)
			{
				Wheel->direction = Cmd->Direction;
				Wheel->Velocity = Cmd->Velocity;
			}

			else
			{
				Wheel->direction = Cmd->Direction;
				Wheel->Velocity = Cmd->Velocity;
			}
		}

		if (Wheel->id == EAST)
		{
			Cmd->Status = OLD;
		}
    }

    else 
    {
 		if (Cmd->LineFollow)
		{			
			LS_MASK = (PRT2DR | IS_LN_MSK);
					
			switch (LS_MASK)
			{
				case EW_LS1_ON:
					if (Wheel->id == WEST)
					{
		   				Wheel->Velocity = Cmd->VS6;
		   			}
			   			
		   			else
		   			{
		   				Wheel->Velocity = Cmd->VP6;
		   			}
			   			
		   			Cmd->END_SENSOR = TRUE;
		    	break;
				    
		    	case EW_LS12_ON:
		    		if (Wheel->id == WEST)
					{
		   				Wheel->Velocity = Cmd->VS5;
		   			}
			   			
		   			else
		   			{
		   				Wheel->Velocity = Cmd->VP5;
		   			}
			   			
		   			Cmd->END_SENSOR = FALSE;
		    	break;

		   		case EW_LS2_ON:
		   			if (Wheel->id == WEST)
					{
		   				Wheel->Velocity = Cmd->VS4;
		   			}
			   			
		   			else
		   			{
		   				Wheel->Velocity = Cmd->VP4;
		   			}
			   			
		   			Cmd->END_SENSOR = FALSE;
		    	break;
				   
		    	case EW_LS23_ON:
		    		if (Wheel->id == WEST)
					{
		   				Wheel->Velocity = Cmd->VS3;
		   			}
			   			
		   			else
		   			{
		   				Wheel->Velocity = Cmd->VP3;
		   			}
			   			
		   			Cmd->END_SENSOR = FALSE;
		   		break;

		   		case EW_LS3_ON:
		   			if (Wheel->id == WEST)
					{
		   				Wheel->Velocity = Cmd->VS2;
		   			}
			   			
		   			else
		   			{
		   				Wheel->Velocity = Cmd->VP2;
		   			}
			   			
		   			Cmd->END_SENSOR = FALSE;
			    	break;
				    
		    	case EW_LS34_ON:
		    		if (Wheel->id == WEST)
					{
		   				Wheel->Velocity = Cmd->VS1;
		   			}
			   			
		   			else
		   			{
		   				Wheel->Velocity = Cmd->VP1;
		   			}
			   			
		   			Cmd->END_SENSOR = FALSE;
		    	break;

				case EW_LS4_ON:
		   			Wheel->Velocity = Cmd->Velocity;
		   			Cmd->END_SENSOR = FALSE;
		    	break;
			    
		    	case EW_LS45_ON:
		    		if (Wheel->id == WEST)
					{
		   				Wheel->Velocity = Cmd->VP1;
		   			}
			   			
		   			else
		   			{
		   				Wheel->Velocity = Cmd->VS1;
		   			}
			   			
		   			Cmd->END_SENSOR = FALSE;
		    	break;

	   			case EW_LS5_ON:
	   				if (Wheel->id == WEST)
					{
	   					Wheel->Velocity = Cmd->VP2;
	   				}
		   				
	   				else
		   			{
		   				Wheel->Velocity = Cmd->VS2;
		   			}
		   				
	   				Cmd->END_SENSOR = FALSE;
	    		break;
			    	
	    		case EW_LS56_ON:
	    			if (Wheel->id == WEST)
					{
	   					Wheel->Velocity = Cmd->VP3;
	   				}
		   				
	   				else
		   			{
		   				Wheel->Velocity = Cmd->VS3;
		   			}
		   				
	   				Cmd->END_SENSOR = FALSE;
	    		break;

	   			case EW_LS6_ON:
	   				if (Wheel->id == WEST)
					{
	   					Wheel->Velocity = Cmd->VP4;
	   				}
		   				
	   				else
		   			{
		   				Wheel->Velocity = Cmd->VS4;
		   			}
		   				
	   				Cmd->END_SENSOR = FALSE;
	    		break;
			    	
	    		case EW_LS67_ON:
	    			if (Wheel->id == WEST)
					{
	   					Wheel->Velocity = Cmd->VP5;
	   				}
		   				
	   				else
		   			{
		   				Wheel->Velocity = Cmd->VS5;
		   			}
		   				
		   			Cmd->END_SENSOR = FALSE;
	    		break;

	   			case EW_LS7_ON:
	   				if (Wheel->id == WEST)
					{
	   					Wheel->Velocity = Cmd->VP6;
	   				}
		   				
	   				else
		   			{
		   				Wheel->Velocity = Cmd->VS6;
		   			}
		   				
		   			Cmd->END_SENSOR = TRUE;
	    		break;
		    		
	    		case ALL_OFF:
	    			if (Cmd->END_SENSOR)
	    			{
	    				Wheel->Velocity = 0;
	    				Cmd->Velocity = 0;
	    			}
						
					else
					{
	    				Wheel->Velocity = Wheel->Velocity;
	    				Cmd->END_SENSOR = FALSE;
	    			}
	    		break;

				default:
		   				Wheel->Velocity = Wheel->Velocity;
	    				Cmd->END_SENSOR = FALSE;
			}//End Switch
		}
		
		Motor(Wheel->id, Wheel->Velocity, Wheel->direction);
	}
	
	return;
}// End ControlWheel