#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules
#include <stdlib.h>
#include "motorcontroller.h"

///////////////////////////////////////////////////////////////////////////////

// PSoC initialization routine

void initPSoC(void)
{

// Initialize the UART receiver
// No parity

	RX8_1_Start(RX8_PARITY_NONE);
	TX8_2_Start(TX8_PARITY_NONE);

// Enable PWM8_1 for motor 0 (left-side motor)

	PWM8_1_DisableInt();
	PWM8_1_WritePulseWidth(0);
	PWM8_1_Start();

// Enable PWM8_2 for motor 1 (right-side motor)

	PWM8_2_DisableInt();
	PWM8_2_WritePulseWidth(0);
	PWM8_2_Start();
	
//	Stop the 16 bit encoder counters

    Counter16_1_Stop();
    Counter16_2_Stop();	
	
// Enable timer interrupts needed for scheduling
	Timer16_1_EnableInt();
	Timer16_1_Start();
	Timer8_1_Start();

// Enable all interrupts
	M8C_EnableGInt;
	
	return;
}

////////////////////////////////////////////////////////////////////////

void Motor(int MotorCode, int PWMvalue, BOOL MotorDirection)
{										// PWMvalue 0-255
   if (MotorCode == EAST) 				// East motor
   {
	   if(MotorDirection == FWD)		// Forward
	   {
		   PRT1DR |= EMC_R_ON;
		   PRT1DR &= EMC_B_OFF;
	   }

	   else 							// Backward
	   {
		   PRT1DR &= EMC_R_OFF;
		   PRT1DR |= EMC_B_ON;
	   }

	   PWM8_2_WritePulseWidth(PWMvalue);
   }

   else 								// West motor
   {
	   if(MotorDirection == FWD)		// Forward
	   {
		   PRT1DR |= WMC_R_ON;
		   PRT1DR &= WMC_B_OFF;
	   }

	   else 							// Backward
	   {
		   PRT1DR &= WMC_R_OFF;
		   PRT1DR |= WMC_B_ON;
	   }

	   PWM8_1_WritePulseWidth(PWMvalue);
   }

   return;
}

//////////////////////////////////////////////////////////////////////

// Stop Function
void StopMotor(int MotorCode)
{
	if (MotorCode == WEST)				// Stop West Motor
	{
		PRT1DR &= WMC_OFF;
	}

	if (MotorCode == EAST)				// Stop East Motor
	{
		PRT1DR &= EMC_OFF;
	}

	if (MotorCode == BOTH)				// Stop both motors
	{
		PRT1DR &= BOTH_OFF;
	}

	PWM8_1_WritePulseWidth(0);
	PWM8_2_WritePulseWidth(0);

	return;
}
